<div class="pagination">
  <ul>
    {% if query.get_page() > 1 %}
      <li class="prev"><a href="./?{{ update_querystring(request.query_string, 'page', query.get_page() - 1) }}">Previous</a></li>
    {% else %}
      <li class="prev disabled"><a href="#">Previous</a></li>
    {% endif %}
    {% if query.get_pages() > query.get_page() %}
      <li class="next"><a href="./?{{ update_querystring(request.query_string, 'page', query.get_page() + 1) }}">Next</a></li>
    {% else %}
      <li class="next disabled"><a href="#">Next</a></li>
    {% endif %}
  </ul>
</div>
