html, body {
  background-color: #fff;
}
body {
  padding-top: 60px;
}
.container > footer p {
  text-align: center; /* center align it with the container */
}

/* The white background content wrapper */
.content {
  background-color: #fff;
  -webkit-border-radius: 0 0 6px 6px;
     -moz-border-radius: 0 0 6px 6px;
          border-radius: 0 0 6px 6px;
}

/* modeladmin filters */
form.modeladmin-filters { margin: 0; }
form.modeladmin-filters ul { list-style-type: none; }
form.modeladmin-filters select { margin-right: 10px; }

div.form-wrapper { width: 100%; overflow: auto; }

form textarea { height: 150px; }
form input[type="text"].datetime-widget { width: 100px; margin-right: 10px; }

div.sidebar li.active { font-weight: bold; }

table.table-striped td.links .inline { margin: 0; }
table.table-striped td.links .inline li { display: inline-block; }
table.table-striped td.links .inline li a {
  border-right: 1px solid #DDDDDD;
  padding-right: 5px;
}
table.table-striped td.links .inline li:last-child a {
  border-right: medium none;
}

/* helpers */
.hidden { display: none; }
