{% extends "admin/models/base.html" %}

{% block extra_script %}
  {{ super() }}
  <script type="text/javascript">
    $(function() {
      var admin_filter = new Admin.ModelAdminFilter()
        , extra = null;

      admin_filter.init();

      {% for qf_s, filter_idx, qf_v, filter_val in active_filters %}
        admin_filter.add_filter_request(
          '{{ qf_s }}', {{ filter_idx }}, '{{ qf_v }}', {{ filter_val|tojson|safe }}
        );
      {% endfor %}
    });
  </script>
{% endblock %}
